/************************************************************************/
/* snowalbedo.c                                                         */
/*                                                                      */
/* Albedoof snow, executable.                                           */
/*                                                                      */
/* Author: Arve Kylling, NILU, Norway.                                  */ 
/*                                                                      */
/*----------------------------------------------------------------------*/
/* Copyright (C) 1997 Arve Kylling                                      */
/*                                                                      */
/* This program is free software; you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 1, or (at your option)  */
/* any later version.                                                   */
/*                                                                      */
/* This program is distributed in the hope that it will be useful,      */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY of FITNESS FOR A PARTICULAR PURPOSE. See the         */
/* GNU General Public License for more details.                         */
/*                                                                      */
/* To obtain a copy of the GNU General Public License write to the      */
/* Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,   */
/* USA.                                                                 */
/*----------------------------------------------------------------------*/
/************************************************************************/

  
/*  @26c@                

@code{integrate} calculates the integral between limits x_min and x_max
by interpolating the data points (x[i], y[i]) with natural cubic
splines or linear interpolation. x_min and x_max are the minimum
and maximum values of the first column in the input_file. The 
x-values in the first column must be in ascending order.

The different options to @code{integrate} are displayed when executing:

@example
integrate -h
@end example

    @c26@ */




#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <ascii.h>
#include <numeric.h>
#include <getopt.h>
#include "../src_f/f77-uscore.h"

#define PROGRAM "snowalbedo"
#define VERSION "1.0"



static void print_usage (char *name)
{
  fprintf (stderr, "\nusage:   %s input_file\n\n", name);
  fprintf (stderr, "%s calculates the integral between limits x_min and x_max\n", PROGRAM);
  fprintf (stderr, "by interpolating the data points (x[i], y[i]) with natural cubic\n");
  fprintf (stderr, "splines (default) or linear interpolation. x_min and x_max are the minimum\n");
  fprintf (stderr, "and maximum values of the first column in the input_file. The \n");
  fprintf (stderr, "x-values in the first column must be in ascending order.\n\n");
  fprintf (stderr, "Optional arguments:\n");
  fprintf (stderr, "  -l            linear interpolation between data points\n");
  fprintf (stderr, "  -p            turn of printing of messages\n\n");
  fprintf (stderr, "Input-file: ASCII-file, blank delimited\n");
  fprintf (stderr, "              x1  y1  z1  ...\n");
  fprintf (stderr, "              x2  y2  z2  ...\n\n");
  fprintf (stderr, "For example  %s -p ../test/integrate_test.in  will create:\n", PROGRAM);
  fprintf (stderr, "               5.000000e-01    1.000000e+00    5.000000e-01\n");
}

void snowalbedo(double omega, double tau, double gg, double surface_albedo, double umu0,
		       double* albedo_diffuse, double* albedo_direct) 
{
/* Function: snowalbedo                                                   @30_30i@ */
/* Description:                                                                    */
/*  Calculate the diffuse and direct albedo as formulated by Wiscombe and Warren,  */
/*  Journal of the Atmospheric Sciences, vol, 37, 2712-2733, 1980. Equation        */
/*  numbers below refer to equations in their paper.                               */
/*                                                                                 */
/* Parameters:                                                                     */
/*  double omega:      snow single scattering albedo                               */
/*  double tau:        snow optical depth                                          */
/*  double gg:         snow assymmetry factor                                      */
/*  double surface_albedo: albedo of underlying surface                            */
/*  double umu0:       cosine of solar zenith angle                                */
/*  double albedo_diffuse: diffuse snow albedo, Eq. 6                              */
/*  double albedo_direct: direct snow albedo, Eq. 3                                */
/*                                                                                 */
/* Return value:                                                                   */
/*  None                                                                           */
/*                                                                                 */
/* Example:                                                                        */
/* Files:                                                                          */
/* Known bugs:                                                                     */
/* Author: Arve Kylling, arve.kylling@nilu.no                                      */
/*                                                                        @i30_30@ */

  double F77_FCN (dei) (double* x);
  double omegas, taus, ggs;
  double as, bs, chi, PP, gamma, QQP, QQM, QQ;
  double arg, dei1, dei2, dei3;

  /* Delta-Eddington scaling */
  ggs       = gg/(1.+gg);                           /* Eq. 2a */
  omegas    = ((1.-gg*gg)*omega)/(1.-gg*gg*omega);  /* Eq. 2b */
  taus      = (1.-gg*gg*omega)*tau;                 /* Eq. 2c */

  /* Definitions following Eq. 3. */
  as        = 1-omegas*ggs;
  bs        = ggs/as;
  chi       = sqrt(3*as*(1.-omegas));
  PP        = 2.*chi/(3*as);
  gamma     = (1.-surface_albedo)/(1.+surface_albedo);
  QQM       = (gamma-PP)*exp(-chi*tau);
  QQP       = (gamma+PP)*exp(+chi*tau);
  QQ        = (1.+PP)*QQP - (1.-PP)*QQM;
    
  /* Direct snow albedo, Eq. 3 */
  *albedo_direct = (2*(PP*(1.-gamma+omegas*bs) + 
		      omegas*(1.+bs)*(gamma*chi*umu0-PP)/(1.-chi*chi*umu0*umu0))*exp(-taus/umu0) 
		    - omegas*bs*(QQP - QQM)
		    + omegas*(1.+bs)*(QQP/(1.+chi*umu0) - QQM/(1.-chi*umu0) )
		    )/QQ;

  /* Diffuse snow albedo, Eq. 6 */
  arg = -taus;                     dei1 = F77_FCN (dei) (&arg);
  arg = -(1.+chi)*taus;            dei2 = F77_FCN (dei) (&arg);
  arg = -(1.-chi)*taus;            dei3 = F77_FCN (dei) (&arg);
  *albedo_diffuse = (2*PP*( (1.-gamma+omegas*bs)*(1.-taus)-(gamma*omegas)*(1.+bs)/(1.-omegas))*exp(-taus)
    		     - 2*PP*( omegas*(1.+bs)*(2./(chi*chi)+(gamma*taus)/(1-omegas))+
			      			      (1.-gamma+omegas*bs)*taus*taus)*dei1
		     + 2.*omegas*(1.+bs)/(chi*chi)*(QQP*(dei2 + chi - log(1.+chi)) -
							QQM*(dei3 - chi - log(fabs(1.-chi))))
		     - omegas*bs*(QQP-QQM)
		     )/QQ;

}


int main (int argc, char **argv)
{
  char filename[FILENAME_MAX+200] = "";
  int print =1;
  int rows=0;
  int status=0;
  int linear=0;
  int k=0;
  double *x=NULL, *y=NULL;
  
  int max_columns=0, min_columns=0;
  double **data=NULL;

  char *dummy=NULL;
  double a, b, ans;
  int c=0;
  double omega; 
  double tau;
  double gg; 
  double surface_albedo, umu0; 
  double albedo_diffuse;
  double albedo_direct;
  double LWD, Qext, r_mean, rho_ice;

  rho_ice = 0.917; /* The density of ice, in g cm-3. */
  LWD     = 10.;   /* Equivalent depth of liquid water in snowpack, g cm-2. */
  Qext    = 2.01;
  r_mean  = 0.050;  /* Mean snow grain radius. */
  surface_albedo = 0.03;
  umu0 = 0.5;

  while ((c=getopt (argc, argv, "a:c?hl:pr:u:")) != EOF)  {
    switch(c)  {
    case 'a':   
      surface_albedo = strtod (optarg, &dummy);
      break;
    case 'l':   
      LWD = strtod (optarg, &dummy);
      break;
    case 'r':   
      r_mean = strtod (optarg, &dummy);
      r_mean = r_mean*1.e-06; /* Convert from um to cm */
      break;
    case 'u':   
      umu0 = strtod (optarg, &dummy);
      break;
    case 'p':  /* no print messages */  
      print = 0;
      break;   
    case 'h':
      print_usage (argv[0]);
      return (-1);
      break;
    case '?':
      print_usage (argv[0]);
      return (-1);
      break;
    default:
      print_usage (argv[0]);
      return (-1);
    }
  }
      
  /* check number of remaining command line arguments */
  if (argc - optind != 0)  {
    print_usage (argv[0]);
    return -1;
  }
  
  omega = 0.999;
  gg    = 0.9;
  tau   = (3.*LWD*Qext)/(4.*r_mean*rho_ice);

  snowalbedo( omega, tau, gg, surface_albedo, umu0, &albedo_diffuse, &albedo_direct);

  if ( print ) {
    fprintf (stderr, "Direct albedo:  %8.4f\nDiffuse albedo: %8.4f\n",
	     albedo_direct, albedo_diffuse);
  }

  return status;

  strcpy (filename, argv[optind]);
	  
  /* read input file */
  if ( print ) {
    if ( linear ) {
      fprintf (stderr, "Using linear interpolation\n");
    }
    else {
      fprintf (stderr, "Using qubic spline interpolation\n");
    }
    fprintf (stderr, "Results reported for all columns\n");
    fprintf (stderr, " ... reading data from file \"%s\" ...\n", filename);
  }

  /* read file */
  status = ASCII_file2double (filename, &rows, &max_columns, &min_columns, &data);
  if (status!=0) {
    fprintf (stderr, " ... error %d opening file %s\n", status, filename);
    return status;
  }

  if ( print ) {
    fprintf (stderr, " ... counted %d data points\n", rows);
  }
  if ( rows == 0) { 
    fprintf (stderr, "%d data points, aborting.\n", rows);
    status = 99;
    ans = 0.0;
    printf("%13.6e\n",ans);
    return status;
  }

  if (max_columns!=min_columns) {
    fprintf (stderr, " !! ATTENTION !! Inconsistent number of columns\n");
    fprintf (stderr, "     min = %d, max =%d\n", min_columns, max_columns);
  }
  
  if (min_columns<2) {
    fprintf (stderr, " ... ending, too few columns\n");
    return 0;
  }

  for (k=1; k<min_columns; k++) {

    x = ASCII_column (data, rows, 0);
    y = ASCII_column (data, rows, k);

    if ( linear ) {
      ans = integrate(x,y,rows);
    }
    else {
      a = x[0];
      b = x[rows-1];
      integrate_spline(x,y,rows, a, b,&ans);
    }
  if ( k ==1 && print ) {
    fprintf (stderr, "Results:\n");
  }

    if ( k > 1 ) {   printf("   "); } 
    printf("%13.6e",ans);
  }
  printf("\n");

  ASCII_free_double (data, rows);
  free(x);
  free(y);
  
  return 0;
}
